Info<< "Reading field p_rgh\n" << endl;
volScalarField p_rgh
(
    IOobject
    (
        "p_rgh",
        runTime.timeName(),
        mesh,
        IOobject::MUST_READ,
        IOobject::AUTO_WRITE
    ),
    mesh
);

Info<< "Reading field U\n" << endl;
volVectorField U
(
    IOobject
    (
        "U",
        runTime.timeName(),
        mesh,
        IOobject::MUST_READ,
        IOobject::AUTO_WRITE
    ),
    mesh
);

#include "createPhi.H"

Info<< "Constructing multiSpecie multiphase model\n" << endl;
twoPhaseReactingMixtureThermo mixture(mesh);    

volScalarField& alpha1(mixture.alpha1());
volScalarField& alpha2(mixture.alpha2());

Info<< "Reading thermophysical properties\n" << endl;

volScalarField p = mixture.p();
volScalarField T = mixture.T();
volScalarField& rho1 = mixture.thermo1().rho();
const volScalarField& psi1 = mixture.thermo1().psi();
volScalarField& rho2 = mixture.thermo2().rho();
const volScalarField& psi2 = mixture.thermo2().psi();

volScalarField rho
(
    IOobject
    (
        "rho",
        runTime.timeName(),
        mesh,
        IOobject::READ_IF_PRESENT,
        IOobject::AUTO_WRITE
    ),
    alpha1*rho1 + alpha2*rho2
);

dimensionedScalar pMin
(
    "pMin",
    dimPressure,
    mixture
);

mesh.setFluxRequired(p_rgh.name());


#include "readGravitationalAcceleration.H"
#include "readhRef.H"
#include "gh.H"


// Mass flux
// Initialisation does not matter because rhoPhi is reset after the
// alpha1 solution before it is used in the U equation.
surfaceScalarField rhoPhi
(
    IOobject
    (
        "rhoPhi",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    fvc::interpolate(rho)*phi
);

volScalarField dgdt
(
    pos(alpha2)*fvc::div(phi)/max(alpha2, scalar(0.0001))
);

// Construct interface from alpha1 distribution
interfaceProperties interface(alpha1, U, mixture);

// Construct compressible turbulence model
autoPtr<compressible::turbulenceModel> turbulence
(
    compressible::turbulenceModel::New(rho, U, rhoPhi, mixture)
);

Info<< "Creating field kinetic energy K\n" << endl;
volScalarField K("K", 0.5*magSqr(U));

basicMultiComponentMixture& composition1 = mixture.thermo1().composition();
basicMultiComponentMixture& composition2 = mixture.thermo2().composition();

PtrList<volScalarField>& Y1 = composition1.Y();
PtrList<volScalarField>& Y2 = composition2.Y();

word inertSpecie1(mixture.thermo1().lookup("inertSpecie"));
word inertSpecie2(mixture.thermo2().lookup("inertSpecie"));

// Set the turbulence into the reaction model
mixture.reaction1()->setTurbulence(turbulence());
mixture.reaction2()->setTurbulence(turbulence());

multivariateSurfaceInterpolationScheme<scalar>::fieldTable fields1;
multivariateSurfaceInterpolationScheme<scalar>::fieldTable fields2;

forAll(Y1, i)
{
    fields1.add(Y1[i]);
}
forAll(Y2, i)
{
    fields2.add(Y2[i]);
}

fields1.add(mixture.thermo1().he());
fields2.add(mixture.thermo2().he());

Info << "\nCreating mass transfer model" << endl;

wordList massTransferActivceSpecies
(
    mixture.subDict("interfaceMassTransfer").lookup("activeSpecies")
);

PtrList<massTransferModel> speciesMasstransferModels(massTransferActivceSpecies.size());
PtrList<volScalarField> speciesmDots(massTransferActivceSpecies.size());

forAll(massTransferActivceSpecies, i)
{
    Info << "\nReading mass transfer models for " 
         << massTransferActivceSpecies[i] << ":" << endl;

    speciesMasstransferModels.set
    (
        i,
        new massTransferModel
        (
            mesh, 
            mixture, 
            U, 
            massTransferActivceSpecies[i]
        )
    );

    // Checking for presence on specie in both phases
    speciesMasstransferModels[i].speciesCheck
    (
        composition1.species(),
        massTransferActivceSpecies[i],
        alpha1.name()
     );
    speciesMasstransferModels[i].speciesCheck
    (
        composition2.species(), 
        massTransferActivceSpecies[i],
        alpha2.name()
    );

    speciesmDots.set
    (
        i,
        new volScalarField
        (
            IOobject
            (
                IOobject::groupName("mDot", massTransferActivceSpecies[i]),
                runTime.timeName(),
                mesh,
                IOobject::NO_READ,
                IOobject::AUTO_WRITE
            ),
            mesh,
            dimensionedScalar("mDot", dimMass/dimVol/dimTime, 0.0)
        )
    );
}

volScalarField vDotAlphaTotal
(
    IOobject
    (
        "vDotTotal",
        runTime.timeName(),
        mesh
    ),
    mesh,
    dimensionedScalar("vDotTotal", dimless/dimTime, 0.0)
);


volScalarField vDotPTotal
(
    IOobject
    (
        "vDotPTotal",
        runTime.timeName(),
        mesh
    ),
    mesh,
    dimensionedScalar("vDotPTotal", dimless/dimTime, 0.0)
);


volScalarField dQ
(
    IOobject
    (
        "dQ",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::AUTO_WRITE
    ),
    mixture.dQ()
);

