/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011-2012 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "Henry.H"
#include "addToRunTimeSelectionTable.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

defineTypeNameAndDebug(Henry, 0);
defineRunTimeSelectionTable(Henry, subModel);
addToRunTimeSelectionTable(equilibriumModel, Henry, equilibriumModel);

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Henry::Henry
(
    const word& type,
    const fvMesh& mesh,
    const dictionary& dict,
    const volVectorField& U,
    const word& specieName
)
:
    equilibriumModel(mesh, dict, U, specieName),
    specie_(specieName),
    dict_(dict)
{
    word gasPhase 
        = 
         dict_.subDict("interfaceMassTransfer")
        .subDict(specie_)
        .subDict("Henry")
        .lookup("gasPhase");

    gasPhase_ = gasPhase;
}


// * * * * * * * * * * * * * * * * * Selectors * * * * * * * * * * * * * * * //

autoPtr<Henry> Henry::New
(
    const fvMesh& mesh,
    const dictionary& dict,
    const volVectorField& U,
    const word& specieName
)
{
    // get model name, but do not register the dictionary
    // otherwise it is registered in the database twice
    const word modelType
    (
        IOdictionary
        (
            IOobject
            (
                "thermophysicalProperties",
                U.time().constant(),
                U.db(),
                IOobject::MUST_READ_IF_MODIFIED,
                IOobject::NO_WRITE,
                false
            )
        )
        .subDict("interfaceMassTransfer")
        .subDict(specieName)
        .subDict("Henry")
        .lookup("correlation")
    );

    Info<< "Selecting Henry's constant correlation model " << modelType << endl;

    subModelConstructorTable::iterator cstrIter =
        subModelConstructorTablePtr_->find(modelType);

    if (cstrIter == subModelConstructorTablePtr_->end())
    {
        FatalErrorInFunction
            << "Unknown Henry's constant correlation type "
            << modelType << nl << nl
            << "Valid Henry's constant correlation types:" << endl
            << subModelConstructorTablePtr_->sortedToc()
            << exit(FatalError);
    }

    return autoPtr<Henry>
    (
        cstrIter()(mesh, dict, U, specieName)
    );
}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

tmp<volScalarField> Henry::equilibriumConcentration
(
    const volScalarField& Yi1,
    const volScalarField& Yi2
)
{
    gasPhaseCheck(Yi1.name(), Yi2.name());

    const volScalarField& p = Yi1.db().lookupObject<volScalarField>("p");

    if(IOobject::groupName(specie_, gasPhase_) == Yi1.name())
        return henryCoeff()*p*Yi1;
    else
        return henryCoeff()*p*Yi2;
}


tmp<volScalarField> Henry::currentConcentration
(
    const volScalarField& Yi1,
    const volScalarField& Yi2,
    const volScalarField& rho1,
    const volScalarField& rho2
)
{
    gasPhaseCheck(Yi1.name(), Yi2.name());

    if(IOobject::groupName(specie_, gasPhase_) == Yi1.name())
        return Yi2*rho2;
    else
        return Yi1*rho1;
}


scalar Henry::MTDirection
(
    const volScalarField& Yi1,
    const volScalarField& Yi2
)
{
    gasPhaseCheck(Yi1.name(), Yi2.name());

    if(IOobject::groupName(specie_, gasPhase_) == Yi1.name())
        return -1.0;
    else
        return 1.0;
}


void Henry::gasPhaseCheck
(
    const word& phaseOne,
    const word& phaseTwo
)
{
    if
    (
        IOobject::groupName(specie_, gasPhase_) != phaseOne &&
        IOobject::groupName(specie_, gasPhase_) != phaseTwo
    )
    {
        FatalErrorInFunction
            << "Unknown gas phase name "
            << gasPhase_ << nl << nl
            << "Valid phase names:" << endl
            << nameExpansion(phaseOne)[1] << nl 
            << nameExpansion(phaseTwo)[1] << nl
            << exit(FatalError);
    }
}


void Henry::correctEquilibrium()
{
    equilibriumModel::correctEquilibrium();
}


bool Henry::read()
{
    return true;
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// ************************************************************************* //
