/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011-2012 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Namespace
    Foam::Henrys

Group
    grpIcoRASTurbulence

Description
    Namespace for Henry's equilibrium based interface mass transfer models.
    Returns specie equilibrium molar fraction based on the Henry's 
    equilibrium model.

Class
    Foam::Henry

SourceFiles
    Henry.C

\*---------------------------------------------------------------------------*/

#ifndef Henry_H
#define Henry_H

#include "equilibriumModel.H"
#include "IOdictionary.H"
#include "autoPtr.H"
#include "runTimeSelectionTables.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
                           Class Henry Declaration
\*---------------------------------------------------------------------------*/

class Henry
:
    public equilibriumModel
{

protected:

    // Protected data
    const word& specie_;
    const dictionary& dict_;
    word gasPhase_;


private:

    // Private Member Functions

        //- Disallow default bitwise copy construct
        Henry(const Henry&);

        //- Disallow default bitwise assignment
        void operator=(const Henry&);


public:

    //- Runtime type information
    TypeName("Henry");


    // Declare run-time constructor selection table

        declareRunTimeSelectionTable
        (
            autoPtr,
            Henry,
            subModel,
            (
                const fvMesh& mesh,
                const dictionary& dict,
                const volVectorField& U,
                const word& specieName
            ),
            (mesh, dict, U, specieName)
        );


    // Constructors

        //- Construct from components
        Henry
        (
            const word& type,
            const fvMesh& mesh,
            const dictionary& dict,
            const volVectorField& U,
            const word& specieName = equilibriumModel::typeName
        );


    // Selectors

        //- Return a reference to the selected RAS model
        static autoPtr<Henry> New
        (
            const fvMesh& mesh,
            const dictionary& dict,
            const volVectorField& U,
            const word& specieName = equilibriumModel::typeName
        );


    //- Destructor
    virtual ~Henry()
    {}


    // Member Functions

        //- Return the specie equilibrium values  
        virtual tmp<volScalarField> equilibriumConcentration
        (
            const volScalarField& Yi1,
            const volScalarField& Yi2
        );

        //- Return the specie current values 
        virtual tmp<volScalarField> currentConcentration
        (
            const volScalarField& Yi1,
            const volScalarField& Yi2,
            const volScalarField& rho1,
            const volScalarField& rho2
        );

        //- Return the mass transfer direction, basd on the gas phase
        //  if the main phase is gas it will be -1, since all the 
        //  calculations are done based on the liquid phase
        virtual scalar MTDirection
        (
            const volScalarField& Yi1,
            const volScalarField& Yi2
        );

        //- Checking for gas phase 
        virtual void gasPhaseCheck
        (
            const word& phaseOne,
            const word& phaseTwo
        );

        //- Return the specie equilibrium coefficient  
        virtual tmp<volScalarField> henryCoeff() = 0;

        //- Correct mass transfer model
        virtual void correctEquilibrium();

        //- Read dictionary
        virtual bool read();
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
