/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011-2013 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "equilibriumModel.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

defineTypeNameAndDebug(equilibriumModel, 0);
defineRunTimeSelectionTable(equilibriumModel, equilibriumModel);

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

equilibriumModel::equilibriumModel
(
    const fvMesh& mesh,
    const dictionary& dict,
    const volVectorField& U,
    const word& specieName
)
:
    regIOobject
    (
        IOobject
        (
            specieName,
            U.time().constant(),
            U.db(),
            IOobject::NO_READ,
            IOobject::NO_WRITE
        )
    ),
    mesh_(mesh)
{}


// * * * * * * * * * * * * * * * * * Selectors * * * * * * * * * * * * * * * //

autoPtr<equilibriumModel> equilibriumModel::New
(
    const fvMesh& mesh,
    const dictionary& dict,
    const volVectorField& U,
    const word& specieName
)
{
    // get model name, but do not register the dictionary
    // otherwise it is registered in the database twice
    const word equilibriumType
    (
        IOdictionary
        (
            IOobject
            (
                "thermophysicalProperties",
                U.time().constant(),
                U.db(),
                IOobject::MUST_READ_IF_MODIFIED,
                IOobject::NO_WRITE,
                false
            )
        )
        .subDict("interfaceMassTransfer")
        .subDict(specieName)
        .lookup("equilibriumModel")
    );

    Info << "Selecting interface equilibrium model type " 
         << equilibriumType << endl;

    equilibriumModelConstructorTable::iterator cstrIterEqui =
        equilibriumModelConstructorTablePtr_->find(equilibriumType);

    if (cstrIterEqui == equilibriumModelConstructorTablePtr_->end())
    {
        FatalErrorInFunction
            << "Unknown interface equlibrium model type "
            << equilibriumType << nl << nl
            << "Valid interface equlibrium model types:" << endl
            << equilibriumModelConstructorTablePtr_->sortedToc()
            << exit(FatalError);
    }

    return autoPtr<equilibriumModel>
    (
        cstrIterEqui()(mesh, dict, U, specieName)
    );
}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

Pair<word> equilibriumModel::nameExpansion(const word& combinedName)
{
    std::stringstream fullName(combinedName);
    std::string segment;
    Pair<word> names;

    int iter = 0;

    while(std::getline(fullName, segment, '.'))
    {
        names[iter] = segment;
        iter++;
    }

    return names;
}

void equilibriumModel::correctEquilibrium()
{}

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// ************************************************************************* //
