/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011-2013 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Namespace
    Foam::equilibriumModels

Description
    Namespace for inteface mass transfer equilibrium models.

Class
    Foam::equilibriumModel

Description
    Abstract base class for mass transfer models

SourceFiles
    equilibriumModel.C

\*---------------------------------------------------------------------------*/

#ifndef equilibriumModel_H
#define equilibriumModel_H

#include "typeInfo.H"
#include "volFields.H"
#include "dimensionedScalar.H"
#include "Pair.H"
#include "nearWallDist.H"
#include "autoPtr.H"
#include "runTimeSelectionTables.H"
#include "fvCFD.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

// Forward declarations
class fvMesh;

/*---------------------------------------------------------------------------*\
                           Class equilibriumModel Declaration
\*---------------------------------------------------------------------------*/

class equilibriumModel
:
    public regIOobject
{

protected:

    const fvMesh& mesh_;


private:

    // Private Member Functions

        //- Disallow default bitwise copy construct
        equilibriumModel(const equilibriumModel&);

        //- Disallow default bitwise assignment
        void operator=(const equilibriumModel&);


public:

    //- Runtime type information
    TypeName("equilibriumModel");


    // Declare run-time New selection table

        // For equilibrium model
        declareRunTimeNewSelectionTable
        (
            autoPtr,
            equilibriumModel,
            equilibriumModel,
            (
                const fvMesh& mesh,
                const dictionary& dict,
                const volVectorField& U,
                const word& specieName
            ),
            (mesh, dict, U, specieName)
        );


    // Constructors

        //- Construct from components
        equilibriumModel
        (
            const fvMesh& mesh,
            const dictionary& dict,
            const volVectorField& U,
            const word& specieName = typeName
        );


    // Selectors

        //- Return a reference to the selected mass transfer model
        static autoPtr<equilibriumModel> New
        (
            const fvMesh& mesh,
            const dictionary& dict,
            const volVectorField& U,
            const word& specieName = typeName
        );


    //- Destructor
    virtual ~equilibriumModel()
    {}


    // Member Functions

        //- Return the specie equilibrium values 
        virtual tmp<volScalarField> equilibriumConcentration
        (
            const volScalarField& Yi1,
            const volScalarField& Yi2
        ) = 0;

        //- Return the specie current values 
        virtual tmp<volScalarField> currentConcentration
        (
            const volScalarField& Yi1,
            const volScalarField& Yi2,
            const volScalarField& rho1,
            const volScalarField& rho2
        ) = 0;

        //- Return the mass transfer direction 
        virtual scalar MTDirection
        (
            const volScalarField& Yi1,
            const volScalarField& Yi2
        ) = 0;

        //- Correct mass transfer model
        virtual void correctEquilibrium() = 0;

        //- Return the first and second parts of a combined name 
        Pair<word> nameExpansion(const word& combinedName);

        //- Read dictionary
        virtual bool read() = 0;

        //- Default dummy write function
        virtual bool writeData(Ostream&) const
        {
            return true;
        }
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
