/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011-2013 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "massTransferModel.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

massTransferModel::massTransferModel
(
    const fvMesh& mesh,
    const dictionary& dict,
    const volVectorField& U,
    const word& specieName
)
:
    mesh_(mesh),
    interfaceArea_(interfaceAreaModel::New(mesh, dict)),
    equilibrium_(equilibriumModel::New(mesh, dict, U, specieName)),
    kinetics_(kineticModel::New(mesh, dict, specieName))
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

Pair<tmp<volScalarField> > massTransferModel::vDot
(
    const scalar& W,
    const volScalarField& Yi1,
    const volScalarField& Yi2,
    const volScalarField& rho1,
    const volScalarField& rho2,
    const volScalarField& rhoi1,
    const volScalarField& rhoi2,
    const volScalarField& alpha
)
{  
    volScalarField pCoeff(1.0/rhoi1 - 1.0/rhoi2);
    volScalarField alphaCoeff(1.0/rhoi1 - alpha*(1.0/rhoi1 - 1.0/rhoi2));
    tmp<volScalarField> mDot_ = this->mDot(W, Yi1, Yi2, rho1, rho2, alpha);

    return Pair<tmp<volScalarField> >
    (
        alphaCoeff*mDot_(),
        pCoeff*mDot_()
    );
}


tmp<volScalarField> massTransferModel::mDot
(
    const scalar& W,
    const volScalarField& Yi1,
    const volScalarField& Yi2,
    const volScalarField& rho1,
    const volScalarField& rho2,
    const volScalarField& alpha
)
{
    dimensionedScalar molecularWeight 
    (
        "molecularWeight", 
        dimensionSet( 1, 0, 0, 0, -1, 0, 0), 
        W/1000.0
    );

    // The mass transfered from phase 1 to phase 2
    return
        kinetics_->massTransferCoeff()
      * interfaceArea_->interfaceArea(alpha)
      * (equilibrium_->equilibriumConcentration(Yi1, Yi2)*molecularWeight 
      - equilibrium_->currentConcentration(Yi1, Yi2, rho1, rho2))
      * equilibrium_->MTDirection(Yi1, Yi2);    
}


volScalarField massTransferModel::massToMole
(
    const scalar& i,
    const basicSpecieMixture& composition
)
{
    volScalarField moleFraction(composition.Y(i));

    moleFraction *= 0.0;

    moleFraction = composition.Y(i)/composition.W(i)*composition.W();

    return moleFraction;
}


volScalarField massTransferModel::specieDensity
(
    const label& i,
    const volScalarField& p,
    const volScalarField& T,
    const basicSpecieMixture& composition
)
{
    volScalarField specieDensity
    (
        IOobject
        (
            "specieDensity",
            mesh_.time().timeName(),
            mesh_
        ),
        mesh_,
        dimensionedScalar("specieDensity", dimensionSet(1,-3,0,0,0,0,0), 0.0)
    );

    forAll(specieDensity, celli)
    {
        specieDensity[celli] = composition.rho(i, p[celli], T[celli]);
    }

    // Boundary correction
    forAll(mesh_.boundary(), patchi)
    {
        forAll(mesh_.boundary()[patchi], facei)
        {
            label faceCelli = mesh_.boundary()[patchi].faceCells()[facei];

            specieDensity.boundaryFieldRef()[patchi][facei] = specieDensity[faceCelli];
        }
    }

    return specieDensity;
}


scalar massTransferModel::speciesCheck
(
    const wordList& species,
    const word& activeSpecie,
    const word& phaseName
)
{
    bool check(false);
    scalar inPhaseSpecieID(-1);

    forAll(species, i)
    {
        if(species[i] == activeSpecie)
        {
            check = true;
            inPhaseSpecieID = i;
        }
    }

    if(!check)
    {
        FatalErrorInFunction
            << "Active mass transfer specie should be present in both phases\n"
            << "Specie " << activeSpecie << " is not defined in the "
            << nameExpansion(phaseName)[1] << " phase" << endl
            << exit(FatalError);
    }

    return inPhaseSpecieID;
}


Pair<word> massTransferModel::nameExpansion(const word& combinedName)
{
    std::stringstream fullName(combinedName);
    std::string segment;
    Pair<word> names;

    int iter = 0;

    while(std::getline(fullName, segment, '.'))
    {
        names[iter] = segment;
        iter++;
    }

    return names;
}


void massTransferModel::correctMassTransfer()
{}

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// ************************************************************************* //
