/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011-2013 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Namespace
    Foam::massTransferModels

Description
    Namespace for compressible multiphase inteface mass transfer models.

Class
    Foam::massTransferModel

Description
    Abstract base class for mass transfer models

SourceFiles
    massTransferModel.C

\*---------------------------------------------------------------------------*/

#ifndef massTransferModel_H
#define massTransferModel_H

#include "twoPhaseReactingMixtureThermo.H"
#include "typeInfo.H"
#include "volFields.H"
#include "dimensionedScalar.H"
#include "Pair.H"
#include "nearWallDist.H"
#include "autoPtr.H"
#include "runTimeSelectionTables.H"
#include "fvCFD.H"
#include "interfaceAreaModel.H"
#include "equilibriumModel.H"
#include "kineticModel.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

// Forward declarations
class fvMesh;

/*---------------------------------------------------------------------------*\
                           Class massTransferModel Declaration
\*---------------------------------------------------------------------------*/

class massTransferModel
{

protected:

    const fvMesh& mesh_;
    autoPtr<interfaceAreaModel> interfaceArea_;
    autoPtr<equilibriumModel> equilibrium_;
    autoPtr<kineticModel> kinetics_;

private:

    // Private Member Functions

        //- Disallow default bitwise copy construct
        massTransferModel(const massTransferModel&);

        //- Disallow default bitwise assignment
        void operator=(const massTransferModel&);


public:

    // Constructors

        //- Construct from components
        massTransferModel
        (
            const fvMesh& mesh,
            const dictionary& dict,
            const volVectorField& U,
            const word& specieName /*= typeName*/
        );


    //- Destructor
    virtual ~massTransferModel()
    {}


    // Member Functions

        //- Return the volumetric mass transfer rates for alpha and P equations
        Pair<tmp<volScalarField> > vDot
        (
            const scalar& W,                // Specie molar wight
            const volScalarField& Yi1,      // Specie mass fraction in phase one
            const volScalarField& Yi2,      // Specie mass fraction in phase two
            const volScalarField& rho1,     // Phase one density
            const volScalarField& rho2,     // Phase two density
            const volScalarField& rhoi1,    // Specie i density in phase one
            const volScalarField& rhoi2,    // Specie i density in phase two
            const volScalarField& alpha     // Phase one volume fraction
        );

        //- Return the mass transfer rates 
        tmp<volScalarField> mDot
        (
            const scalar& W,
            const volScalarField& Yi1,
            const volScalarField& Yi2,
            const volScalarField& rho1,
            const volScalarField& rho2,
            const volScalarField& alpha
        );
        
        // Conversion from mass fraction to mole fraction 
        volScalarField massToMole
        (
            const scalar& i,
            const basicSpecieMixture& composition
        );

        // Calculating specie density 
        volScalarField specieDensity
        (
            const label& i,
            const volScalarField& p,
            const volScalarField& T,
            const basicSpecieMixture& composition
        );

        // Checking for existance of specie in the phase 
        scalar speciesCheck
        (
            const wordList& species,
            const word& activeSpecie,
            const word& phaseName
        );

        //- Return the first and second parts of a combined name 
        Pair<word> nameExpansion(const word& combinedName);

        //- Correct mass transfer model
        virtual void correctMassTransfer();

        //- Read dictionary
        virtual bool read() 
        {
            return true;
        }

        //- Default dummy write function
        virtual bool writeData(Ostream&) const
        {
            return true;
        }
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
