/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2013-2015 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "twoPhaseReactingMixtureThermo.H"
#include "gradientEnergyFvPatchScalarField.H"
#include "mixedEnergyFvPatchScalarField.H"
#include "IFstream.H"


// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
    defineTypeNameAndDebug(twoPhaseReactingMixtureThermo, 0);
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::twoPhaseReactingMixtureThermo::twoPhaseReactingMixtureThermo
(
    const fvMesh& mesh
)
:
    rhoThermo(mesh, word::null),
    twoPhaseMixture(mesh, *this),
    reaction1_(combustionModels::rhoCombustionModel::New(mesh, "combustionProperties", phase1Name())),
    reaction2_(combustionModels::rhoCombustionModel::New(mesh, "combustionProperties", phase2Name()))
{
    {
        volScalarField T1(IOobject::groupName("T", phase1Name()), reaction1_->thermo().T());
        T1.write();
    }

    {
        volScalarField T2(IOobject::groupName("T", phase2Name()), reaction2_->thermo().T());
        T2.write();
    }

    reaction1_->thermo().validate(phase1Name(), "e");
    reaction2_->thermo().validate(phase2Name(), "e");

    correct();
}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::twoPhaseReactingMixtureThermo::~twoPhaseReactingMixtureThermo()
{}


// * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * * //

void Foam::twoPhaseReactingMixtureThermo::correct()
{
    reaction1_->thermo().he() = reaction1_->thermo().he(reaction1_->thermo().p(), reaction1_->thermo().T());
    reaction1_->thermo().correct();

    reaction2_->thermo().he() = reaction2_->thermo().he(reaction2_->thermo().p(), reaction2_->thermo().T());
    reaction2_->thermo().correct();

    mu_ = alpha1()*reaction1_->thermo().mu() + alpha2()*reaction2_->thermo().mu();
    psi_ = alpha1()*reaction1_->thermo().psi() + alpha2()*reaction2_->thermo().psi();
    alpha_ = alpha1()*reaction1_->thermo().alpha() + alpha2()*reaction2_->thermo().alpha();
    rho_ = alpha1()*reaction1_->thermo().rho() + alpha2()*reaction2_->thermo().rho();
}


void Foam::twoPhaseReactingMixtureThermo::reactionCorrect()
{
    reaction1_->correct();
    reaction2_->correct();
}


bool Foam::twoPhaseReactingMixtureThermo::incompressible() const
{
    return reaction1_->thermo().incompressible() && reaction2_->thermo().incompressible();
}


bool Foam::twoPhaseReactingMixtureThermo::isochoric() const
{
    return reaction1_->thermo().isochoric() && reaction2_->thermo().isochoric();
}


Foam::tmp<Foam::volScalarField> Foam::twoPhaseReactingMixtureThermo::he
(
    const volScalarField& p,
    const volScalarField& T
) const
{
    return alpha1()*reaction1_->thermo().he(p, T) + alpha2()*reaction2_->thermo().he(p, T);
}


Foam::tmp<Foam::scalarField> Foam::twoPhaseReactingMixtureThermo::he
(
    const scalarField& p,
    const scalarField& T,
    const labelList& cells
) const
{
    return
        scalarField(alpha1(), cells)*reaction1_->thermo().he(p, T, cells)
      + scalarField(alpha2(), cells)*reaction2_->thermo().he(p, T, cells);
}


Foam::tmp<Foam::scalarField> Foam::twoPhaseReactingMixtureThermo::he
(
    const scalarField& p,
    const scalarField& T,
    const label patchi
) const
{
    return
        alpha1().boundaryField()[patchi]*reaction1_->thermo().he(p, T, patchi)
      + alpha2().boundaryField()[patchi]*reaction2_->thermo().he(p, T, patchi);
}


Foam::tmp<Foam::volScalarField> Foam::twoPhaseReactingMixtureThermo::hc() const
{
    return alpha1()*reaction1_->thermo().hc() + alpha2()*reaction2_->thermo().hc();
}


Foam::tmp<Foam::scalarField> Foam::twoPhaseReactingMixtureThermo::THE
(
    const scalarField& h,
    const scalarField& p,
    const scalarField& T0,
    const labelList& cells
) const
{
    NotImplemented;
    return T0;
}


Foam::tmp<Foam::scalarField> Foam::twoPhaseReactingMixtureThermo::THE
(
    const scalarField& h,
    const scalarField& p,
    const scalarField& T0,
    const label patchi
) const
{
    NotImplemented;
    return T0;
}


Foam::tmp<Foam::volScalarField> Foam::twoPhaseReactingMixtureThermo::Cp() const
{
    return alpha1()*reaction1_->thermo().Cp() + alpha2()*reaction2_->thermo().Cp();
}


Foam::tmp<Foam::scalarField> Foam::twoPhaseReactingMixtureThermo::Cp
(
    const scalarField& p,
    const scalarField& T,
    const label patchi
) const
{
    return
        alpha1().boundaryField()[patchi]*reaction1_->thermo().Cp(p, T, patchi)
      + alpha2().boundaryField()[patchi]*reaction2_->thermo().Cp(p, T, patchi);
}


Foam::tmp<Foam::volScalarField> Foam::twoPhaseReactingMixtureThermo::Cv() const
{
    return alpha1()*reaction1_->thermo().Cv() + alpha2()*reaction2_->thermo().Cv();
}


Foam::tmp<Foam::scalarField> Foam::twoPhaseReactingMixtureThermo::Cv
(
    const scalarField& p,
    const scalarField& T,
    const label patchi
) const
{
    return
        alpha1().boundaryField()[patchi]*reaction1_->thermo().Cv(p, T, patchi)
      + alpha2().boundaryField()[patchi]*reaction2_->thermo().Cv(p, T, patchi);
}


Foam::tmp<Foam::volScalarField> Foam::twoPhaseReactingMixtureThermo::gamma() const
{
    return alpha1()*reaction1_->thermo().gamma() + alpha2()*reaction2_->thermo().gamma();
}


Foam::tmp<Foam::scalarField> Foam::twoPhaseReactingMixtureThermo::gamma
(
    const scalarField& p,
    const scalarField& T,
    const label patchi
) const
{
    return
        alpha1().boundaryField()[patchi]*reaction1_->thermo().gamma(p, T, patchi)
      + alpha2().boundaryField()[patchi]*reaction2_->thermo().gamma(p, T, patchi);
}


Foam::tmp<Foam::volScalarField> Foam::twoPhaseReactingMixtureThermo::Cpv() const
{
    return alpha1()*reaction1_->thermo().Cpv() + alpha2()*reaction2_->thermo().Cpv();
}


Foam::tmp<Foam::scalarField> Foam::twoPhaseReactingMixtureThermo::Cpv
(
    const scalarField& p,
    const scalarField& T,
    const label patchi
) const
{
    return
        alpha1().boundaryField()[patchi]*reaction1_->thermo().Cpv(p, T, patchi)
      + alpha2().boundaryField()[patchi]*reaction2_->thermo().Cpv(p, T, patchi);
}


Foam::tmp<Foam::volScalarField> Foam::twoPhaseReactingMixtureThermo::CpByCpv() const
{
    return
        alpha1()*reaction1_->thermo().CpByCpv()
      + alpha2()*reaction2_->thermo().CpByCpv();
}


Foam::tmp<Foam::scalarField> Foam::twoPhaseReactingMixtureThermo::CpByCpv
(
    const scalarField& p,
    const scalarField& T,
    const label patchi
) const
{
    return
        alpha1().boundaryField()[patchi]*reaction1_->thermo().CpByCpv(p, T, patchi)
      + alpha2().boundaryField()[patchi]*reaction2_->thermo().CpByCpv(p, T, patchi);
}


Foam::tmp<Foam::volScalarField> Foam::twoPhaseReactingMixtureThermo::nu() const
{
    return mu()/(alpha1()*reaction1_->thermo().rho() + alpha2()*reaction2_->thermo().rho());
}


Foam::tmp<Foam::scalarField> Foam::twoPhaseReactingMixtureThermo::nu
(
    const label patchi
) const
{
    return
        mu(patchi)
       /(
            alpha1().boundaryField()[patchi]*reaction1_->thermo().rho(patchi)
          + alpha2().boundaryField()[patchi]*reaction2_->thermo().rho(patchi)
        );
}


Foam::tmp<Foam::volScalarField> Foam::twoPhaseReactingMixtureThermo::kappa() const
{
    return alpha1()*reaction1_->thermo().kappa() + alpha2()*reaction2_->thermo().kappa();
}


Foam::tmp<Foam::scalarField> Foam::twoPhaseReactingMixtureThermo::kappa
(
    const label patchi
) const
{
    return
        alpha1().boundaryField()[patchi]*reaction1_->thermo().kappa(patchi)
      + alpha2().boundaryField()[patchi]*reaction2_->thermo().kappa(patchi);
}


Foam::tmp<Foam::volScalarField> Foam::twoPhaseReactingMixtureThermo::kappaEff
(
    const volScalarField& alphat
) const
{
    return
        alpha1()*reaction1_->thermo().kappaEff(alphat)
      + alpha2()*reaction2_->thermo().kappaEff(alphat);
}


Foam::tmp<Foam::scalarField> Foam::twoPhaseReactingMixtureThermo::kappaEff
(
    const scalarField& alphat,
    const label patchi
) const
{
    return
        alpha1().boundaryField()[patchi]*reaction1_->thermo().kappaEff(alphat, patchi)
      + alpha2().boundaryField()[patchi]*reaction2_->thermo().kappaEff(alphat, patchi)
    ;
}


Foam::tmp<Foam::volScalarField> Foam::twoPhaseReactingMixtureThermo::alphaEff
(
    const volScalarField& alphat
) const
{
    return
        alpha1()*reaction1_->thermo().alphaEff(alphat)
      + alpha2()*reaction2_->thermo().alphaEff(alphat);
}


Foam::tmp<Foam::scalarField> Foam::twoPhaseReactingMixtureThermo::alphaEff
(
    const scalarField& alphat,
    const label patchi
) const
{
    return
        alpha1().boundaryField()[patchi]*reaction1_->thermo().alphaEff(alphat, patchi)
      + alpha2().boundaryField()[patchi]*reaction2_->thermo().alphaEff(alphat, patchi)
    ;
}


Foam::tmp<Foam::volScalarField> Foam::twoPhaseReactingMixtureThermo::rho
() const
{
    return alpha1()*reaction1_->thermo().rho() + alpha2()*reaction2_->thermo().rho();
}


Foam::tmp<Foam::volScalarField> Foam::twoPhaseReactingMixtureThermo::dQ
() const
{
    return alpha1()*reaction1_->Qdot() + alpha2()*reaction2_->Qdot();
}


Foam::tmp<Foam::volScalarField> Foam::twoPhaseReactingMixtureThermo::effDiffusivity
(
    const volScalarField& Yi, 
    const volScalarField& rho,
    const volScalarField& mut,
    const word& phaseName
)
{
    const dictionary& thermophysicalProperties = Yi.db().lookupObject<IOdictionary>
    (
       IOobject::groupName("thermophysicalProperties", phaseName)
    );

    dictionary thermoDict
    (
        IFstream
        (
            fileName(thermophysicalProperties.lookup("foamChemistryThermoFile")).expand()
        )()
    );

    scalar diffCoeff
    (
        readScalar
        (
            thermoDict.subDict(Yi.member()).subDict("transport").lookup("diffCoeff")
        )
    );

    dimensionedScalar diffCoeffDim("diffCoeff", dimensionSet( 0, 2, -1, 0, 0, 0, 0), diffCoeff);

    return mut + diffCoeffDim*rho;
}

// ************************************************************************* //
